/*
 **  EPIwm
 **  Copyright (C) 1999 Julien Mulot, Thomas Poindessous, Stephane Rault
 **
 **  This program is free software; you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation; either version 2 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **
 ** mouse_events.c
 **
 ** Made by (Poindessous Thomas)
 ** Login   <poinde_t@epita.fr>
 **
 ** Started on  Wed Feb 24 15:19:04 1999 Poindessous Thomas
 ** Last update Wed May 19 21:02:10 1999 Thomas Poindessous
 */

#include "epiwm.h"

#define	TIME_RELEASE	400
#define	TIME_TWO_CLICKS	1000
#define TIME_CHANGE_WORKSPACE 0
#define WORKSPACE_PERCENT 1

void			handle_mouse()
{
   Window		child;
   Window		root;
   int			root_x;
   int			root_y;
   int			win_x;
   int			win_y;
   unsigned int		status;
   t_menu_pos		**p_menu;
   
   if (gl_menu.menu_task == gl_event.xbutton.window)
     menu_task_click(gl_event.xmotion.x, gl_event.xmotion.y);
   else
     {
	p_menu = &menu_pos;
	while ((*p_menu) != NULL)
	  {
	     if (((*p_menu)->win) == gl_event.xbutton.window)
	       break;
	     p_menu = &((*p_menu)->next);
	  }
	if ((*p_menu))
	  {
	     XQueryPointer(gl_dpy, gl_event.xbutton.window, &root, &child,
			   &root_x, &root_y, &win_x, &win_y, &status);
	     menu_click(*p_menu, win_x, win_y, status);
	  }
	else
	  menu_display(gl_event.xbutton.window, gl_event.xbutton.x, 
		       gl_event.xbutton.y, gl_event.xbutton.button);
     }
}

void		mouse_on_menu(menu)
t_menu_pos	*menu;
{
   int		i;
   int		elt_pos;
   int		text_width;
   t_menu_apps	*p_menu_apps;
   
   p_menu_apps = *(menu->pos);
   elt_pos = gl_event.xmotion.y / 20 + 1;
   for (i = 0; i < elt_pos; i++)
     {
	p_menu_apps = p_menu_apps->next_elt;
	if (!p_menu_apps)
	  break;
     }
   if (p_menu_apps)
     {
	if (gl_menu.current_elt != &(p_menu_apps->elt))
	  {
	     XClearArea(gl_dpy, menu->win, gl_menu.width - 14, 0,
			14, menu->height, False);
	     draw_menu(menu->pos, &menu);
	     text_width = XTextWidth(gl_menu.font, p_menu_apps->elt,
				     strlen(p_menu_apps->elt));
	     XDrawString(gl_dpy, menu->win, gl_menu.font_active,
			 (gl_menu.width - text_width) / 2,
			 elt_pos * 20, p_menu_apps->elt,
			 strlen(p_menu_apps->elt));
	     if (p_menu_apps->menu)
	       draw_triangle(menu->win, gl_menu.width - 14, 
			     elt_pos * 20 - 8, "active");
	     gl_menu.current_elt = &(p_menu_apps->elt);
	  }
     }
}

void		catch_win_mouse()
{
   Window	child;
   Window	root;
   int		root_x;
   int		root_y;
   int		win_x;
   int		win_y;
   unsigned int	status;
   t_win	*temp;
   
   XQueryPointer(gl_dpy, gl_scr_info.root_win, &root, &child,
		 &root_x, &root_y, &win_x, &win_y, &status);
   temp = search_elt(child);
   if (temp)
     if (child == temp->parent)
       window_state(temp, "active");
}

void			mouse_enter_win()
{
   t_mouse_move_win	list_icon;
   
   if (gl_event.xcrossing.window != gl_scr_info.root_win)
     {
	list_icon.temp = search_elt(gl_event.xcrossing.window);
	if (list_icon.temp)
	  {
	     if (list_icon.temp->iconify_win == gl_event.xcrossing.window)
	       mouse_on_icon(gl_event.xcrossing.window);
	     else if (list_icon.temp->parent == gl_event.xcrossing.window)
	       window_state(list_icon.temp, "active");
	  }
     }
}

void			mouse_leave_win()
{
   t_mouse_move_win	list_icon;
   t_menu_pos		**menu;
   
   if (gl_event.xcrossing.window != gl_scr_info.root_win)
     {
	list_icon.temp = search_elt(gl_event.xcrossing.window);
	if (list_icon.temp)
	  {
	     if (list_icon.temp->iconify_win == gl_event.xcrossing.window)
	       mouse_out_icon(gl_event.xcrossing.window);
	     else if (list_icon.temp->parent == gl_event.xcrossing.window)
	       {
		  window_state(list_icon.temp, "inactive");
		  gl_epiwin.current = NULL;
	       }
	  }
	else
	  {
	     menu = &menu_pos;
	     while ((*menu) != NULL)
	       {
		  if ((*menu)->win == gl_event.xcrossing.window)
		    break;
		  menu = &((*menu)->next);
	       }
	     if (((*menu) != NULL) && ((*menu)->here))
	       draw_menu((*menu)->pos, menu);
	  }
     }
}

void			mouse_press()
{
   XWindowAttributes	attrib;
   XButtonEvent		*button;
   t_win		*temp;
   
   button = &gl_event.xbutton;
   gl_win_move.bef = gl_win_move.win;
   gl_win_move.win = button->window;
   gl_win_move.mouse_x = button->x_root;
   gl_win_move.mouse_y = button->y_root;
   gl_win_move.win_x = button->x;   
   gl_win_move.win_y = button->y;
   /*   printf("mouse press %d %d\n", button->x, button->y);*/
   gl_win_move.time_press = button->time;
   temp = search_elt(button->window);
   if (temp)
     {
	XGetWindowAttributes(gl_dpy, temp->border_frame , &attrib);
	gl_win_move.width = attrib.width;
	gl_win_move.height = attrib.height;
	XRaiseWindow(gl_dpy, temp->parent);
	gl_epiwin.is_window_changing = 1;
     }
   handle_mouse();
}

void		mouse_on_icon(icon)
Window          icon;
{
   char		*name;
   int		font_direction;
   int		top_size;
   int		bottom_size;
   XCharStruct	dim;
   
   if (!gl_icon.image_active)
     {
	XSetWindowBackground(gl_dpy, icon, gl_icon.color_active.pixel);
	XClearWindow(gl_dpy, icon);
     }
#ifdef	  __Imlib__
   else
     draw_icon_image(icon, "active");
#endif
   XGetIconName(gl_dpy, icon, &name);
   XTextExtents(gl_icon.font, name, strlen(name), &font_direction,
		&top_size, &bottom_size, &dim);
   if ((dim.width + 6) > gl_icon.width)
     {
	XResizeWindow(gl_dpy, icon, dim.width + 6, gl_icon.height);
	draw_icon(icon, dim.width + 6, "active");
     }
   else
     {
	XResizeWindow(gl_dpy, icon, gl_icon.width, gl_icon.height);
	draw_icon(icon, gl_icon.width, "active");
     }
   free(name);
}

void		mouse_out_icon(icon)
Window          icon;
{
   if (!gl_icon.image_active)
     {
	XSetWindowBackground(gl_dpy, icon, gl_icon.color_inactive.pixel);
	XClearWindow(gl_dpy, icon);
     }
#ifdef	  __Imlib__
   else
     draw_icon_image(icon, "inactive");
#endif
   XResizeWindow(gl_dpy, icon, gl_icon.width, gl_icon.height);
   draw_icon(icon, gl_icon.width, "inactive");
}

void			mouse_release()
{
   t_mouse_move_win	list;
   
   list.button = &gl_event.xbutton;
   gl_epiwin.is_window_changing = 0;
   if (!(Button1Mask == (list.button->state & Button1Mask)))
     return ;
   list.temp = search_elt(list.button->window);
   if (!list.temp)
     return ;
   /*   if (list.temp->resize_button == list.button->window)
	mouse_resize(&list);   
	else
	*/
   if (list.temp->button_max == list.button->window)
     mouse_resize_max(&list);
   /*     win_max_width(list.temp);*/
   else if (list.temp->button_iconify == list.button->window)
     mouse_iconify(&list);
   else if (list.temp->iconify_win == list.button->window)
     mouse_uniconify(&list);
   else if (((list.button->time - gl_win_move.time_press) < TIME_RELEASE) &&
	    list.temp->titlebar_win == list.button->window)
     mouse_shade(&list);
   else if (list.temp->titlebar_win == list.button->window)
     mouse_move_win(&list);
   else if (list.temp->border_frame == list.button->window)
     mouse_resize_l_r_d(&list);
   gl_win_move.last = list.button->time;
}

void			mouse_resize(list)
t_mouse_move_win	*list;
{
   XWindowAttributes	attrib;
   int			diffx;
   int			diffy;
   int			diff;
   int			width_inc;
   int			height_inc;

   return ;
   
   width_inc = 1;
   height_inc = 1;
   /*   mouse_resize_win_increment(list->temp->win,
	&width_inc, &height_inc);*/
   width_inc = list->temp->hints.width_inc;
   height_inc = list->temp->hints.height_inc;
   if (width_inc == 0)
     width_inc = 1;
   if (height_inc == 0)
     height_inc = 1;
   XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
   if (list->button->x_root < (attrib.x + attrib.width))
     diffx = -1;
   else
     diffx = 1;
   if (list->button->y_root < (attrib.y + attrib.height))
     diffy = -1;
   else
     diffy = 1;
   XGetWindowAttributes(gl_dpy, list->temp->win, &attrib);
   gl_event.xresizerequest.window = list->temp->win;
   diff = (((int) (list->button->x_root - gl_win_move.mouse_x)) / width_inc)
     * width_inc;
   if (diff < 0)
     diff *= -1;
   if (diffx < 0)
     diff *= -1;
   gl_event.xresizerequest.width = diff + attrib.width;
   diff = ((int) ((list->button->y_root - gl_win_move.mouse_y)) / height_inc)
     * height_inc;
   if (diff < 0)
     diff *= -1;
   if (diffy < 0)
     diff *= -1;
   gl_event.xresizerequest.height = diff + attrib.height;
   if (gl_event.xresizerequest.height < 0)
     gl_event.xresizerequest.height = 1;
   if (gl_event.xresizerequest.width < 0)
     gl_event.xresizerequest.width = 1;
   resize_win();
   if (test_shape(list->temp->win))
     {
	gl_event.xmap.window = list->temp->win;
	map_win();
     }   
}

void			mouse_resize_max(list)
t_mouse_move_win	*list;
{
   XWindowAttributes	attrib;
   
   if ((list->button->x < 0) || (list->button->y < 0) ||
       (list->button->x > gl_epiwin.btn_width)
       || (list->button->y > gl_epiwin.btn_height))
     return ;
   if (list->temp->is_max)
     {
	list->temp->is_max = 0;
	if (list->temp->is_shade == 2)
	  {
	     list->temp->shade_width = list->temp->win_width;
	     list->temp->shade_height = list->temp->win_height;
	     XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
	     window_group_resize(list->temp, list->temp->win_x,
				 list->temp->win_y, list->temp->win_width,
				 gl_epiwin.titlebar.height +
				 2*gl_epiwin.titlebar.border);
	  }
	else
	  {
	     window_group_resize(list->temp, list->temp->win_x,
				 list->temp->win_x, list->temp->win_width,
				 list->temp->win_height);		      
	  }
	send_configure_notify(list->temp);
     }
   else
     {
	XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
	list->temp->win_x = attrib.x;
	list->temp->win_y = attrib.y;
	list->temp->is_max = 1;
	if (list->temp->is_shade == 2)
	  {
	     list->temp->win_width = list->temp->shade_width;
	     list->temp->win_height = list->temp->shade_height;
	     window_group_resize(list->temp, 0, 0,		 
				 gl_scr_info.dpy_width,
				 gl_epiwin.titlebar.height +
				 2*gl_epiwin.titlebar.border);
	  }
	else
	  {
	     XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
	     list->temp->win_width = attrib.width;
	     list->temp->win_height = attrib.height;
	     window_group_resize(list->temp, 0, 0,
				 gl_scr_info.dpy_width,
				 gl_scr_info.dpy_height);
	     send_configure_notify(list->temp);
	     /*
		XMoveWindow(gl_dpy, list->temp->parent, 0, 0);
		gl_event.xresizerequest.window = list->temp->win;
		gl_event.xresizerequest.width = gl_scr_info.dpy_width -
		(gl_epiwin.border_width + attrib.border_width
		+ gl_epiwin.bf_border) * 2;
		gl_event.xresizerequest.height = gl_scr_info.dpy_height -
		(gl_epiwin.border_height + attrib.border_width +
		gl_epiwin.titlebar.border) * 2
		- gl_epiwin.titlebar.height
		- gl_epiwin.bf_border;
		resize_win();
		*/
	  }
     }
   if (test_shape(list->temp->win))
     {
	gl_event.xmap.window = list->temp->win;
	map_win();
     }
}

void			mouse_shade(list)
t_mouse_move_win	*list;
{
   XWindowAttributes	attrib;
   
   if (list->temp->is_shade == 0)
     list->temp->is_shade = 1;
   else if ((list->temp->is_shade == 1)
	    && (gl_win_move.bef == gl_win_move.win))
     {
	if ((list->button->time - gl_win_move.last) < TIME_TWO_CLICKS)
	  {
	     list->temp->is_shade = 2;
	     if (list->temp->is_max)
	       {
		  XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
		  XResizeWindow(gl_dpy, list->temp->parent,
				gl_scr_info.dpy_width,
				gl_epiwin.titlebar.height +
				2*gl_epiwin.titlebar.border);
	       }
	     else
	       {
		  XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
		  list->temp->shade_width = attrib.width;
		  list->temp->shade_height = attrib.height;
		  if (!(list->temp->is_max_width || list->temp->is_max_height))
		    {
		       list->temp->win_x = attrib.x;
		       list->temp->win_y = attrib.y;
		       list->temp->win_width = attrib.width;
		       list->temp->win_height = attrib.height;
		    }
		  XResizeWindow(gl_dpy, list->temp->parent,
				attrib.width, gl_epiwin.titlebar.height +
				2*gl_epiwin.titlebar.border);
	       }
	  }
     }
   else if (list->temp->is_shade == 2)
     list->temp->is_shade = 3;
   else if (list->temp->is_shade == 3)
     {
	if ((list->button->time - gl_win_move.last) < TIME_TWO_CLICKS)
	  {
	     list->temp->is_shade = 0;
	     if (list->temp->is_max)
	       {
		  XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
		  window_group_resize(list->temp, attrib.x, attrib.y,
				      gl_scr_info.dpy_width,
				      gl_scr_info.dpy_height);
		  send_configure_notify(list->temp);
	       }
	     else
	       {
		  XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
		  if (list->temp->is_max_width)
		    window_group_resize(list->temp, attrib.x, attrib.y,
					gl_scr_info.dpy_width,
					list->temp->shade_height);	       
		  else if (list->temp->is_max_height)
		    window_group_resize(list->temp, attrib.x, attrib.y,
					list->temp->shade_width,
					gl_scr_info.dpy_height);
		  else
		    window_group_resize(list->temp, attrib.x, attrib.y,
					list->temp->shade_width,
					list->temp->shade_height);
		  send_configure_notify(list->temp);
	       }
	  }
     }
   else
     list->temp->is_shade = 2;
}

void			mouse_iconify(list)
t_mouse_move_win	*list;
{
   char			*name;
   t_workspace_window	*ww_tmp;
   
   if ((list->button->x >= 0) && (list->button->y >= 0) &&
       (list->button->x <= gl_epiwin.btn_width)
       && (list->button->y <= gl_epiwin.btn_height))
     {
	XGetIconName(gl_dpy, list->temp->parent, &name);
	list->temp->iconify_win = add_iconify_win(list->temp);
	XSetIconName(gl_dpy, list->temp->iconify_win, name);
	free(name);
	XUnmapWindow(gl_dpy, list->temp->parent);
	XMapWindow(gl_dpy, list->temp->iconify_win);
	clientSetState(list->temp, IconicState);
#ifdef	  __Imlib__
	if (gl_icon.image_inactive)
	  draw_icon_image(list->temp->iconify_win, "inactive");
#endif
	draw_icon(list->temp->iconify_win, gl_icon.width, "inactive");
	ww_tmp = workspace_window_search(gl_epiwin.current_workspace,
					 list->temp);
	if (ww_tmp)
	  {
	     ww_tmp->is_win_mapped = 0;
	     ww_tmp->is_icon_mapped = 1;
	  }
     }
}

void			mouse_uniconify(list)
t_mouse_move_win	*list;
{
   t_workspace_window	*ww_tmp;
   
   XDestroyWindow(gl_dpy, list->temp->iconify_win);
   iconify_rm_elt(list->temp);
   XMapWindow(gl_dpy, list->temp->parent);
   clientSetState(list->temp, NormalState);
   ww_tmp = workspace_window_search(gl_epiwin.current_workspace,
				    list->temp);
   if (ww_tmp)
     {
	ww_tmp->is_win_mapped = 1;
	ww_tmp->is_icon_mapped = 0;
     } 
}

void			mouse_move_win(list)
t_mouse_move_win	*list;
{
   int			x_root;
   int			y_root;

   XClearWindow(gl_dpy, gl_scr_info.root_win);
   x_root = list->button->x_root - gl_win_move.win_x;
   y_root = list->button->y_root - gl_win_move.win_y;
   XMoveWindow(gl_dpy, list->temp->parent, x_root, y_root);
   clientSetState(list->temp, NormalState);
   send_configure_notify(list->temp);
}

void			mouse_resize_l_r_d(list)
t_mouse_move_win	*list;
{
   /*
      XWindowAttributes	attrib;
      int			height;
      int			width;
      int			width_inc;
      int			height_inc;
      */
   
   return ;
   /*
      width_inc = 1;
      height_inc = 1;
      mouse_resize_win_increment(list->temp->win, &width_inc, &height_inc);
      XGetWindowAttributes(gl_dpy, list->temp->border_frame, &attrib);
      if (gl_win_move.win_x <= gl_epiwin.border_width)
      {
      XGetWindowAttributes(gl_dpy, list->temp->parent, &attrib);
      width = (((int) (gl_win_move.mouse_x - list->button->x_root))
      / width_inc) * width_inc;
      if (attrib.x - width >= 0)
      if (-width < attrib.width)
      XMoveWindow(gl_dpy, list->temp->parent,
      attrib.x - width, attrib.y);
      else
      XMoveWindow(gl_dpy, list->temp->parent,
      attrib.x + attrib.width - gl_epiwin.border_width
      - gl_epiwin.border_width,
      attrib.y);
      else
      XMoveWindow(gl_dpy, list->temp->parent,
      0, attrib.y);
      XGetWindowAttributes(gl_dpy, list->temp->win, &attrib);
      gl_event.xresizerequest.width = width + attrib.width;
      gl_event.xresizerequest.height = attrib.height;
      }
      else if (gl_win_move.win_x >= (attrib.width - gl_epiwin.border_width ))
      if (gl_win_move.win_y >= (attrib.height - gl_epiwin.border_height))
      {
      mouse_resize(list);
      return ;
      }
      else
      {
      XGetWindowAttributes(gl_dpy, list->temp->win, &attrib);
      width = (((int) (list->button->x - gl_win_move.win_x))
      / width_inc) * width_inc;
      gl_event.xresizerequest.width = width + attrib.width;
      gl_event.xresizerequest.height = attrib.height;
      }
      else if (gl_win_move.win_y >= (attrib.height - gl_epiwin.border_height))
      {
      XGetWindowAttributes(gl_dpy, list->temp->win, &attrib);
      height = (((int) (list->button->y - gl_win_move.win_y)) / height_inc)
      * height_inc;
      gl_event.xresizerequest.width = attrib.width;
      gl_event.xresizerequest.height = height + attrib.height;
      }
      else
      {	
      return ;
      }
      if (gl_event.xresizerequest.height < 0)
      gl_event.xresizerequest.height = 1;
      if (gl_event.xresizerequest.width < 0)
      gl_event.xresizerequest.width = 1;
      gl_event.xresizerequest.window = list->temp->win;
      resize_win();
      if (test_shape(list->temp->win))
      {
      gl_event.xmap.window = list->temp->win;
      map_win();
      }
      */
}


void mouse_on_border_notify()
{
   if (gl_event.xmotion.x_root == gl_scr_info.dpy_width - 1)
     wait_before_change_workspace('r');
   else if (gl_event.xmotion.x_root == 0)
     wait_before_change_workspace('l');
   else if (gl_event.xmotion.y_root == gl_scr_info.dpy_height - 1)
     wait_before_change_workspace('d');
   else if (gl_event.xmotion.y_root == 0)
     wait_before_change_workspace('u');
   else
     XDefineCursor(gl_dpy, gl_scr_info.root_win, gl_epiwin.normal_cursor);
}

void			wait_before_change_workspace(direction)
char			direction;
{
   Window		child;
   Window		root;
   int			root_x;
   int			root_y;
   int			win_x;
   int			win_y;
   unsigned int		status;
   int			i;
   struct timeval	time;
   int			is_still_here;
   
   time.tv_sec = 0;
   time.tv_usec = 10000;
   is_still_here = 0;
   set_cursor_pager(direction);
   if (TIME_CHANGE_WORKSPACE > 0)
     {
	for (i = 0; i < TIME_CHANGE_WORKSPACE; i++)
	  {
	     time.tv_sec = 0;
	     time.tv_usec = 10000;
	     (void) select(1, 0, 0, 0, &time);
	     XQueryPointer(gl_dpy, gl_event.xbutton.window, &root, &child,
			   &root_x, &root_y, &win_x, &win_y, &status);
	     is_still_here = compare_mouse_position(direction, root_x, root_y);
	     if (is_still_here == 0)
	       break;
	  }
	if (is_still_here == 1)
	  go_to_pager(direction, root_x, root_y);
     }
   else
     {
	XQueryPointer(gl_dpy, gl_event.xbutton.window, &root, &child,
		      &root_x, &root_y, &win_x, &win_y, &status);
	go_to_pager(direction, root_x, root_y);
     }
}

void	set_cursor_pager(direction)
char	direction;
{
   if (direction == 'l')
     XDefineCursor(gl_dpy, gl_scr_info.root_win, gl_epiwin.resize_W_cursor);
   if (direction == 'r')
     XDefineCursor(gl_dpy, gl_scr_info.root_win, gl_epiwin.resize_E_cursor);
}

int	compare_mouse_position(direction, root_x, root_y)
char	direction;
int	root_x;
int	root_y;
{
   int	is_still_here;
   
   is_still_here = 0;
   if (direction == 'r')
     if (root_x == gl_scr_info.dpy_width - 1)
       is_still_here = 1;
   if (direction == 'l')
     if (root_x == 0)
       is_still_here = 1;
   return (is_still_here);
}

void	go_to_pager(direction, current_x, current_y)
char	direction;
int	current_x;
int	current_y;
{  
   if ((direction == 'l') && (gl_epiwin.virtual_x > 0))
     {
	gl_epiwin.virtual_x = (gl_epiwin.virtual_x - gl_scr_info.dpy_width *
			       WORKSPACE_PERCENT / 100);
	move_all_windows(gl_scr_info.dpy_width *
			 WORKSPACE_PERCENT / 100, 0);
	XWarpPointer(gl_dpy, gl_scr_info.root_win, gl_scr_info.root_win,
		     0, 0, gl_scr_info.dpy_width, gl_scr_info.dpy_height,
		     current_x + gl_scr_info.dpy_width *
		     WORKSPACE_PERCENT / 100 - 2, current_y);
     }
   else if ((direction == 'r') &&
	    ((gl_epiwin.virtual_x +
	      gl_scr_info.dpy_width) < gl_epiwin.virtual_width))
     {
	gl_epiwin.virtual_x = (gl_epiwin.virtual_x + gl_scr_info.dpy_width *
			       WORKSPACE_PERCENT / 100);
	move_all_windows(-(gl_scr_info.dpy_width *
			   WORKSPACE_PERCENT / 100), 0);
	XWarpPointer(gl_dpy, gl_scr_info.root_win, gl_scr_info.root_win,
		     0, 0, gl_scr_info.dpy_width, gl_scr_info.dpy_height,
		     current_x - gl_scr_info.dpy_width *
		     WORKSPACE_PERCENT / 100 + 2, current_y);
     }
}
/* end troll */

void			move_all_windows(width, height)
int			width;
int			height;
{
   t_win_list		*w_list;
   XWindowAttributes	attrib;
  
   w_list = gl_epiwin.first_win_group_list;
   while (w_list)
     {
	if (w_list->p_t_win && (width != 0 || height != 0))
	  {
	     XGetWindowAttributes(gl_dpy, w_list->p_t_win->parent, &attrib);
	     XMoveWindow(gl_dpy, w_list->p_t_win->parent,
			 attrib.x + width, attrib.y + height);
	  }
	w_list = w_list->next_elt;
     }
}
