/*
 **  EPIwm
 **  Copyright (C) 1999 Julien Mulot, Thomas Poindessous, Stephane Rault
 **
 **  This program is free software; you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation; either version 2 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **
 ** window.h
 ** 
 ** Made by (Poindessous Thomas)
 ** Login   <poinde_t@epita.fr>
 ** 
 ** Started on  Sat Jan 16 16:14:51 1999 Poindessous Thomas
 ** Last update Wed May 12 01:37:25 1999 Julien Mulot
 */

#ifndef   __WINDOW_H__
#define   __WINDOW_H__

#include <X11/cursorfont.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

typedef	struct	s_titlebar
{
#define ALIGN_LEFT 1
#define ALIGN_CENTER 2
#define ALIGN_RIGHT 3
#define TOP 1
#define BOTTOM 2
#define LEFT 3
#define RIGHT 4
   char		*image_active;
   char		*image_inactive;
   char		*align;
   char		grad_active_type;
   char		grad_inactive_type;
   char		border;
   int		height;
   int		width;
  int		space;
  char		position;
#ifdef	  __Imlib__
   ImlibImage	*image;
#endif
   Pixel	*pixels_active;
   Pixel	*pixels_inactive;
   GC		font_active;
   GC		font_inactive;
   XColor	color_active;
   XColor	color_inactive;
   XColor	color_stick;
   XFontStruct	*font;
}		t_titlebar;

typedef struct	s_epiwin
{
   Cursor	iconify_cursor;
   Cursor	normal_cursor;
   Cursor	resize_S_cursor;
   Cursor	resize_N_cursor;
   Cursor	resize_E_cursor;
   Cursor	resize_W_cursor;
   Cursor	resize_NW_cursor;
   Cursor	resize_NE_cursor;
   Cursor	resize_SW_cursor;
   Cursor	resize_SE_cursor;
   Cursor	move_cursor;
   int		border_height;
   int		border_width;
   int		btn_width;
   int		btn_height;
   char		is_window_changing;
   char		opaque_move;
   char		opaque_resize;
   char		bf_border;
   char		is_btn_close;
   char		is_btn_iconify;
   char		is_btn_max;
   char		is_btn_shade;
   char		is_btn_resize;
   short	align_btn_max;
   short	align_btn_close;
   short	align_btn_resize;
   short	align_btn_shade;
   short	align_btn_iconify;
   int		pos_btn_max;
   int		pos_btn_close;
   int		pos_btn_resize;
   int		pos_btn_shade;
   int		pos_btn_iconify;
   XColor	color_active;
   XColor	color_inactive;
   XColor	color_stick;
   XContext	context;
   t_win	*current;
   t_titlebar	titlebar;
   t_workspace	*first_workspace;
   t_workspace	*current_workspace;
   t_win_list	*first_win_group_list;
   int		nb_elt;
   int		virtual_width;
   int		virtual_height;
   int		virtual_x;
   int		virtual_y;
}		t_epiwin;

typedef struct		s_window_btn
{
   int			x;
   int			y;
   int			height;
   int			width;
   char			border;
   unsigned long	bgcolor;
   unsigned long	fgcolor;
   Cursor		cursor;
   Window		parent;
}			t_window_btn;

typedef struct		s_color
{
   int			allocated;
   unsigned char	red;
   unsigned char	green;
   unsigned char	blue;
   unsigned long	pixel;
}			t_color;

Window		add_parent(XWindowAttributes *);
Window		add_resize_btn(Window);
Window		add_resize_max(Window);
Window		add_shade_btn(Window);
Window		add_iconify_btn(Window);
int		test_shape(Window);
void		add_shape(Window, Window);
void		window_build(FILE *);
Window		add_titlebar_win(Window);
Window		add_border_frame(Window);
Window		add_btn(t_window_btn *);
void		draw_titlebar(Window, char *);
void		draw_titlebar_image(Window, char *);
void		window_state(t_win *, char *);
void		check_properties(t_win *);
void		set_titlebar_position(char *);
int		set_title_position();

t_epiwin	gl_epiwin;
t_color		from;
t_color		to;

#endif /* __WINDOW_H__ */
