/*
 **  EPIwm
 **  Copyright (C) 1999 Julien Mulot, Thomas Poindessous, Stephane Rault
 **
 **  This program is free software; you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation; either version 2 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **
 ** window_state.c
 ** 
 ** Made by (Poindessous Thomas)
 ** Login   <poinde_t@epita.fr>
 ** 
 ** Started on  Sat Mar 20 16:42:40 1999 Poindessous Thomas
 ** Last update Tue May 11 00:11:59 1999 Poindessous Thomas
 */

#include "epiwm.h"

void			clientSetState(epi_win, state)
t_win			*epi_win;
int			state;
{
   unsigned long	data[2];
   
   epi_win->state = state;
   data[0] = (unsigned long) state;
   data[1] = (unsigned long) None; /* icon window */
   XChangeProperty(gl_dpy, epi_win->win, EPIWM_WM_STATE, EPIWM_WM_STATE, 32, 
		   PropModeReplace, (unsigned char *) data, 2);
}

void			send_configure_notify(epi_win)
t_win			*epi_win;
{
   XEvent		sevent;
   XWindowAttributes	attrib;
   int			pos_x;
   int			pos_y;
   
   XGetWindowAttributes(gl_dpy, epi_win->win, &attrib);
   sevent.type = ConfigureNotify;
   sevent.xconfigure.display = gl_dpy;
   sevent.xconfigure.event = epi_win->win;
   sevent.xconfigure.window = epi_win->win;
   sevent.xconfigure.width = attrib.width;
   sevent.xconfigure.height = attrib.height;
   sevent.xconfigure.border_width = attrib.border_width;
   sevent.xconfigure.override_redirect = False;
   sevent.xconfigure.above = epi_win->win;
   XGetWindowAttributes(gl_dpy, epi_win->parent, &attrib);
   pos_x = attrib.x + gl_epiwin.border_width + gl_epiwin.bf_border;
   pos_y = attrib.y + gl_epiwin.border_height + gl_epiwin.titlebar.height
     + 2 * gl_epiwin.titlebar.border;
   sevent.xconfigure.x = pos_x;
   sevent.xconfigure.y = pos_y;
   XSendEvent(gl_dpy, epi_win->win, False, StructureNotifyMask, &sevent);
}

void			client_message()
{
   t_win		*temp;
   t_mouse_move_win	*list;
   
   temp = search_elt(gl_event.xclient.window);
   if (temp)
     {
	if (gl_event.xclient.message_type == EPIWM_WM_CHANGE_STATE)
	  if (gl_event.xclient.data.l[0] == IconicState)
	    {
	       list = malloc(sizeof(t_mouse_move_win));
	       list->button = malloc(sizeof(XButtonEvent));
	       list->button->x = 0;
	       list->button->y = 0;
	       list->temp = temp;
	       mouse_iconify(list);
	       free(list->button);
	       free(list);
	    }
     }
}

void			window_group_resize(win_group,
					    parent_x, parent_y,
					    parent_width, parent_height)
t_win			*win_group;
int			parent_x;
int			parent_y;
int			parent_width;
int			parent_height;
{
   XWindowAttributes	attrib;
   
   XGetWindowAttributes(gl_dpy, win_group->parent, &attrib);
   if (parent_height - gl_epiwin.titlebar.height -
       2*gl_epiwin.titlebar.border != 0)
     {
	if (parent_width < (win_group->hints.min_width
			    + 2*(win_group->win_border + gl_epiwin.border_width
				 + gl_epiwin.bf_border)))  
	  parent_width = win_group->hints.min_width
	    + 2*(win_group->win_border + gl_epiwin.border_width
		 + gl_epiwin.bf_border);
	if (parent_height < (win_group->hints.min_height
			     + 2*(win_group->win_border +
				  gl_epiwin.border_height
				  + gl_epiwin.titlebar.border)
			     + gl_epiwin.titlebar.height
			     + gl_epiwin.bf_border))
	  parent_height = win_group->hints.min_height
	    + 2*(win_group->win_border + gl_epiwin.border_height
		 + gl_epiwin.titlebar.border)
	      + gl_epiwin.titlebar.height + gl_epiwin.bf_border;
	if (parent_width > (win_group->hints.max_width
			    + 2*(win_group->win_border + gl_epiwin.border_width
				 + gl_epiwin.bf_border)) &&
	    win_group->hints.max_width != 0)
	  parent_width = win_group->hints.max_width
	    + 2*(win_group->win_border + gl_epiwin.border_width
		 + gl_epiwin.bf_border);
	if (parent_height > (win_group->hints.max_height
			     + 2*(win_group->win_border +
				  gl_epiwin.border_height
				  + gl_epiwin.titlebar.border)
			     + gl_epiwin.titlebar.height + gl_epiwin.bf_border)
	    && win_group->hints.max_height != 0)
	  parent_height = win_group->hints.max_height
	    + 2*(win_group->win_border + gl_epiwin.border_height
		 + gl_epiwin.titlebar.border)
	      + gl_epiwin.titlebar.height + gl_epiwin.bf_border;
     }
   if (parent_width <= attrib.width && parent_width <= attrib.height)
     {
	if (attrib.x == parent_x && attrib.y == parent_y)
	  XResizeWindow(gl_dpy, win_group->parent,
			parent_width, parent_height);
	else
	  XMoveResizeWindow(gl_dpy, win_group->parent,
			    parent_x, parent_y, parent_width, parent_height);
     }   
   XResizeWindow(gl_dpy, win_group->titlebar_win, 
		 parent_width - 2*gl_epiwin.titlebar.border,
		 gl_epiwin.titlebar.height);
   if (parent_height - gl_epiwin.titlebar.height -
       2*gl_epiwin.titlebar.border - gl_epiwin.bf_border > 0)
     {
	XResizeWindow(gl_dpy, win_group->border_frame, 
		      parent_width - 2*gl_epiwin.bf_border,
		      parent_height - gl_epiwin.titlebar.height -
		      2*gl_epiwin.titlebar.border
		      - gl_epiwin.bf_border);
	XResizeWindow(gl_dpy, win_group->win, 
		      parent_width - 2*(gl_epiwin.bf_border
					+ gl_epiwin.border_width
					+ win_group->win_border),
		      parent_height - gl_epiwin.titlebar.height -
		      2*(gl_epiwin.titlebar.border
			 + gl_epiwin.border_height
			 + win_group->win_border) - gl_epiwin.bf_border);
     }
      if (gl_epiwin.is_btn_max)
     XMoveWindow(gl_dpy, win_group->button_max,
		 set_btn_position(win_group->titlebar_win,
				  gl_epiwin.align_btn_max,
				  gl_epiwin.pos_btn_max),
		 (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2);
   if (gl_epiwin.is_btn_iconify)
     XMoveWindow(gl_dpy, win_group->button_iconify,
		 set_btn_position(win_group->titlebar_win,
				  gl_epiwin.align_btn_iconify,
				  gl_epiwin.pos_btn_iconify),
		 (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2);
   if (gl_epiwin.is_btn_shade)
     XMoveWindow(gl_dpy, win_group->button_shade,
		 set_btn_position(win_group->titlebar_win,
				  gl_epiwin.align_btn_shade,
				  gl_epiwin.pos_btn_shade),
		 (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2);
   XMoveWindow(gl_dpy, win_group->win, gl_epiwin.border_width,
	       gl_epiwin.border_height);
   if (!(parent_width <= attrib.width && parent_width <= attrib.height))
     {
	if (attrib.x == parent_x && attrib.y == parent_y)
	  XResizeWindow(gl_dpy, win_group->parent,
			parent_width, parent_height);
	else
	  XMoveResizeWindow(gl_dpy, win_group->parent,
			    parent_x, parent_y, parent_width, parent_height);
     }
   else
     {

     }
    
}

void	window_group_map(win_group)
t_win	*win_group;
{
   if (win_group->is_win_shape)
     add_shape(win_group->win, win_group->parent);
   XMapSubwindows(gl_dpy, win_group->parent);
   XMapSubwindows(gl_dpy, win_group->border_frame);
   XMapSubwindows(gl_dpy, win_group->titlebar_win);	     
   XMapWindow(gl_dpy, win_group->parent);
}

void		clientsendprotocol(wg_tmp, protocol)
t_win		*wg_tmp;
Atom		protocol;
{
   XEvent	event;
   
   event.xclient.type = ClientMessage;
   event.xclient.message_type = EPIWM_WM_PROTOCOLS;
   event.xclient.format = 32;
   event.xclient.display = gl_dpy;
   event.xclient.window = wg_tmp->win;
   event.xclient.data.l[0] = protocol;
   /*
      event.xclient.data.l[1] = time;
      event.xclient.data.l[2] = 0;
      event.xclient.data.l[3] = 0;
      */
   XSendEvent(gl_dpy, wg_tmp->win, False, NoEventMask, &event);
   XSync(gl_dpy, False);
}




